#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>

extern int errno;

void main(int argc, char ** argv) {
  //four more cells for "strace -f -q -etrace=open,execve -o tmp.command.dep"
  char **command = malloc(sizeof(char*)*(argc+6)),*cmd=NULL;
  pid_t pid;
  void *interrupt_signal, *quit_signal;
  int status,index,offset,f_flag=1,flag_cnt=0,i;

  if (argc==1) {
    printf("dep needs the program name you want to trace.\n");
    exit(1);
  }

  //for the help flag, print out the help message.
  if (argc==2 && (strcmp(argv[1],"-?")==0 || strcmp(argv[1],"-help")==0)) {
    printf("Usage: dep [[-debug] [-detail] [-nokid] [-list] [-clear] [-merge] [-move] [-copy]] files\n\n");
    printf("\tdebug:\t\toutput more debug info.\n");
    printf("\tdetail:\t\tdisplay package::file format.\n");
    printf("\tnokid:\t\twon't trace forked child process.\n");
    printf("\tlist:\t\tlist the existing dep. info.\n");
    printf("\tclear:\t\tdelete the dep. file.\n");
    printf("\tmerge:\t\tmerge two dep. files into one.\n");
    printf("\tmove:\t\trename the dep. file.\n");
    printf("\tcopy:\t\tdouplicate a dep. file.\n");
    printf("\n");
    exit(0);
  }

  //if found a flag, then don't do strace, call the wrapper directly.
  if (argv[1][0]=='-' && argv[1][1]!='d' && strcmp(argv[1],"-nokid")!=0) {
    command[0] = (char*)strdup("swrapper.pl");
    for (index=1;index<argc;index++)
      command[index] = argv[index];
    execvp(command[0],command);
    return;
  }
  else {
    offset=0;
    if (argv[1][0]=='-') {
      f_flag &= strcmp(argv[1],"-nokid");
      offset++;
      if (argc>2 && argv[2][0]=='-') {
	f_flag &= strcmp(argv[2],"-nokid");
	offset++;
	if (argc>3 && argv[3][0]=='-') {
	  f_flag &= strcmp(argv[3],"-nokid");
	  offset++;
	}
      }
    }
    flag_cnt=0;
    command[flag_cnt++] = (char*)strdup("strace");
    if (f_flag) {
      command[flag_cnt++] = (char*)strdup("-f");
    }
    command[flag_cnt++] = (char*)strdup("-q");
    command[flag_cnt++] = (char*)strdup("-etrace=open,execve");
    command[flag_cnt++] = (char*)strdup("-o");
    command[flag_cnt] = malloc(sizeof(char)*(strlen(argv[1])+10));
    if (cmd = (char*)strrchr(argv[1+offset],(int)'/')) {
      cmd++;
    }
    else {
      cmd = argv[offset+1];
    }
    sprintf(command[flag_cnt++],"/tmp/tmp.%s.dep",cmd);
    for (index=1;index<argc;index++) {
      command[index+flag_cnt-1] = argv[index+offset];
    }
    
    //fork the child here.
    pid = fork();
    if (pid <0) {
      error(1, errno, "cannot fork");
    }
    else if (pid==0) {
      execvp(command[0],command);
      error(0,errno,"cannot run %s",command[0]);
      _exit(errno==ENOENT?127:126);
    }
    
    interrupt_signal = (void*)signal(SIGINT, SIG_IGN);
    quit_signal = (void*)signal(SIGQUIT, SIG_IGN);
    
    pid = wait(&status);
    if (pid==-1) {
      error(1,errno,"error waiting for child process");
    }
    
    signal(SIGINT,interrupt_signal);
    signal(SIGQUIT,quit_signal);
    
    if (command[0]) {
      free(command[0]);
      command[0] = (char*)strdup("swrapper.pl");
    }

    index=1;
    if (argv[1][0]=='-') {
      command[1] = argv[index];
      index++;
    
      if (argc>2 && argv[2][0]=='-') {
	command[2] = argv[index];
	index++;

	if (argc>3 && argv[3][0]=='-') {
	  command[3] = argv[index];
	  index++;
	}
      }
    }

    command[index] = cmd;
    index++;
    
    for (index;index<argc;index++) {
      command[index] = argv[index];
    }
    

    //call the wrapper and handle error.
    execvp(command[0],command);
    error(0,errno,"cannot run %s",command[0]);
  }
}
